class PeopleErrorsTest < ActionController::TestCase

  tests PeopleController

  test "it returns validation errors" do
    post :create, person: { age: 23 }

    assert_response :unprocessable_entity
    parsed_body = JSON.parse(response.body)
    assert_equal ["errors"], parsed_body.keys
    assert_equal ["name"], parsed_body["errors"].keys
  end

  test "it rejects unpermitted parameters" do
    post :create, person: { updated_at: 27, age: 23 }

    assert_response :bad_request
    parsed_body = JSON.parse(response.body)
    assert_equal ["error", "message"], parsed_body.keys
    assert_equal "unpermitted_parameters", parsed_body["error"]
  end

  test "it requires required parameters" do
    post :create, {}

    assert_response :bad_request
    parsed_body = JSON.parse(response.body)
    assert_equal ["error", "message"], parsed_body.keys
    assert_equal "missing_parameters", parsed_body["error"]
  end

  test "it returns 404 when no record exists" do
    get :show, id: '1234567890'

    assert_response :not_found
    parsed_body = JSON.parse(response.body)
    assert_equal ["error"], parsed_body.keys
    assert_equal "not_found", parsed_body["error"]
  end

  test "it refuses sneaky parameters in filter" do
    person = Person.create(name: 'Nobody', age: 23)
    other_person = Person.create(name: 'Somebody', age: 27)

    get :index, filter: { something: { evil: "in here" } }

    assert_response :bad_request
  end
end

