class PeopleTest < ActionController::TestCase

  tests PeopleController

  test "it can create a record" do
    post :create, person: { name: 'Nobody', age: 23 }

    assert_response :success
    assert Person.find_by(name: 'Nobody').present?
  end

  test "it can read a record" do
    person = Person.create(name: 'Nobody', age: 23)

    get :show, id: person.id.to_s

    assert_response :success
    parsed_body = JSON.parse(response.body)
    assert_equal person.id, parsed_body["id"]
    assert_equal person.name, parsed_body["name"]
  end

  test "it can list all records" do
    person = Person.create(name: 'Nobody', age: 23)
    other_person = Person.create(name: 'Somebody', age: 27)

    get :index

    assert_response :success
    parsed_body = JSON.parse(response.body)
    assert parsed_body.is_a?(Array)
  end

  test "it can list all records matching a filter" do
    person = Person.create(name: 'Nobody', age: 23)
    other_person = Person.create(name: 'Somebody', age: 27)

    get :index, filter: { name: 'Somebody' }

    assert_response :success
    parsed_body = JSON.parse(response.body)
    assert_equal 1, parsed_body.length
  end

  test "it can update a record" do
    person = Person.create(name: 'Nobody', age: 23)

    put :update, id: person.id.to_s, person: { name: 'Nobody 2' }

    assert_response :success
    assert Person.find_by(name: 'Nobody 2').present?
  end

  test "it can destroy a record" do
    person = Person.create(name: 'Nobody', age: 23)

    delete :destroy, id: person.id.to_s

    assert_response :success
  end
end
